@@ -1,21 +1,26 @@
 import time
-from enum import Enum
 from collections.abc import Iterable
+from enum import Enum
+from types import SimpleNamespace
+from typing import List, Dict, Optional, Collection, Union, Tuple, Generator
 
-from requests.packages.urllib3.util import Retry
-from requests.adapters import HTTPAdapter
 from requests import Session
+from requests.adapters import HTTPAdapter
+from requests.packages.urllib3.util import Retry
 
-from d20.Manual.Logger import logging
-from d20.Manual.Exceptions import (ConsoleError,
+from d20.Manual.BattleMap import FileObject
+from d20.Manual.Logger import logging, Logger
+from d20.Manual.Exceptions import (ConsoleError, RPCTimeoutError,
                                    WaitTimeoutError)
-from d20.Manual.Facts import resolveFacts
-from d20.Manual.RPC import (RPCResponseStatus,
-                            RPCCommands,
+from d20.Manual.Facts import resolveFacts, Fact
+from d20.Manual.RPC import (RPCClient, RPCResponseStatus,
+                            RPCCommands, RPCResponse,
                             RPCStreamCommands)
+from d20.Manual.Temporary import PlayerDirectoryHandler
+import d20.Manual.Trackers as Tracker
 
 
-LOGGER = logging.getLogger(__name__)
+LOGGER: Logger = logging.getLogger(__name__)
 
 
 class PlayerState(Enum):
@@ -38,45 +43,50 @@ class ConsoleInterface(object):
             config: The config for this console instance
 
     """
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         try:
-            self._id = kwargs['id']
-            self._cloneID = kwargs.get('cloneID', None)
-            self._directoryHandler = kwargs['directoryHandler']
-            self._rpc = kwargs['rpc_client']
-            self._async = kwargs['asyncData']
-            self._config_ = kwargs.get('config', None)
+            self._id: str = kwargs['id']
+            self._cloneID: Optional[str] = kwargs.get('cloneID', None)
+            self._directoryHandler: PlayerDirectoryHandler = \
+                kwargs['directoryHandler']
+            self._rpc: RPCClient = kwargs['rpc_client']
+            self._async: SimpleNamespace = kwargs['asyncData']
+            self._config_: Union[str, Dict, None] = kwargs.get('config', None)
         except KeyError:
             LOGGER.critical("Expected argument not passed to init",
                             exc_info=True)
             raise
 
-        self._session = None
-        self._sessionConfig = dict()
-        self._sessionRetryConfig = dict()
+        self._session: Optional[Session] = None
+        self._sessionConfig: Dict = dict()
+        self._sessionRetryConfig: Dict = dict()
         # Setup RPC handler for the console
 
-    def __getSession(self, total=5, status_forcelist=None,
-                     backoff_factor=0.2,
-                     method_whitelist=Retry.DEFAULT_METHOD_WHITELIST,
-                     **kwargs):
+    def __getSession(self, total: int = 5,
+                     status_forcelist: Optional[Collection[int]] = None,
+                     backoff_factor: float = 0.2,
+                     method_whitelist: Optional[Collection[str]] =
+                     Retry.DEFAULT_METHOD_WHITELIST,
+                     **kwargs: str) -> Session:
 
         # XXX TODO Retry.DEFAULT_METHOD_WHITELIST is deprecated in favor
         # of Retry.DEFAULT_METHODS_ALLOWED and method_whitelist is deprecated
         # in favor of allowed_methods, both of which do not seem to exist in
         # the version of urllib that gets pulled in
-        retryConfig = Retry(total=total, status_forcelist=status_forcelist,
-                            backoff_factor=backoff_factor,
-                            method_whitelist=method_whitelist,
-                            raise_on_redirect=False, **kwargs)
+        retryConfig: Retry = Retry(total=total,
+                                   status_forcelist=status_forcelist,
+                                   backoff_factor=backoff_factor,
+                                   method_whitelist=method_whitelist,
+                                   raise_on_redirect=False, **kwargs)
 
-        session = Session()
+        session: Session = Session()
 
-        proxies = dict()
-        if 'http_proxy' in self._config_:
-            proxies['http'] = self._config_['http_proxy']
-        if 'https_proxy' in self._config_:
-            proxies['https'] = self._config_['https_proxy']
+        proxies: Dict[str, str] = dict()
+        if isinstance(self._config_, dict):
+            if 'http_proxy' in self._config_:
+                proxies['http'] = self._config_['http_proxy']
+            if 'https_proxy' in self._config_:
+                proxies['https'] = self._config_['https_proxy']
 
         if len(proxies.keys()) > 0:
             session.proxies = proxies
@@ -93,7 +103,7 @@ class ConsoleInterface(object):
         return session
 
     @property
-    def async_(self):
+    def async_(self) -> SimpleNamespace:
         """asyncio support data
 
             This property/method returns a data object that contains three
@@ -114,7 +124,7 @@ class ConsoleInterface(object):
         return self._async
 
     @property
-    def requests(self):
+    def requests(self) -> Session:
         """Configured requests session
 
             This property/method returns a configured Session object
@@ -123,10 +133,10 @@ class ConsoleInterface(object):
             more information
         """
         if self._session is None:
-            self._session = self.__getSession(self._sessionRetryConfig)
+            self._session = self.__getSession(**self._sessionRetryConfig)
         return self._session
 
-    def configureRequestsRetry(self, **kwargs):
+    def configureRequestsRetry(self, **kwargs) -> None:
         """Configuration options for the Retry class
 
             This function allows you to customize the behavior of the Retry
@@ -135,7 +145,7 @@ class ConsoleInterface(object):
         """
         self._sessionRetryConfig = kwargs
 
-    def configureRequestsSession(self, config):
+    def configureRequestsSession(self, config: Dict) -> None:
         """Configuration options for Session object
 
             This function takes a dict with k/v pairs that coorespond to
@@ -156,18 +166,18 @@ class ConsoleInterface(object):
         self._session = None
 
     @property
-    def myDirectory(self):
+    def myDirectory(self) -> str:
         """Returns entity's directory information"""
         return self._directoryHandler.myDir
 
-    def createTempDirectory(self):
+    def createTempDirectory(self) -> str:
         """Creates and returns a temporary directory"""
         return self._directoryHandler.tempdir()
 
-    def _noop(self):
+    def _noop(self) -> None:
         self._rpc.sendAndIgnore(command=RPCCommands.noop)
 
-    def print(self, *args, **kwargs):
+    def print(self, *args, **kwargs) -> None:
         """Prints out stuff, similar to python built-in print
 
             Entities should not use the built-in print since this provides
@@ -177,8 +187,12 @@ class ConsoleInterface(object):
         self._rpc.sendAndIgnore(command=RPCCommands.print,
                                 args={'args': args, 'kwargs': kwargs})
 
-    def addObject(self, object_data, creator, parentObjects,
-                  parentFacts, parentHyps, metadata, encoding):
+    def _addObject(self, object_data: bytes, creator: str,
+                   parentObjects: Optional[List[FileObject]],
+                   parentFacts: Optional[List[Fact]],
+                   parentHyps: Optional[List[Fact]],
+                   metadata: Optional[Dict],
+                   encoding: Optional[str]) -> RPCResponse:
         """Adds an object to the object list
 
             Args:
@@ -186,7 +200,6 @@ class ConsoleInterface(object):
 
             Returns: The object id
         """
-
         if (parentObjects is not None
                 and not isinstance(parentObjects, Iterable)):
             raise ValueError("parent objects must be a list")
@@ -199,7 +212,7 @@ class ConsoleInterface(object):
                 and not isinstance(parentHyps, Iterable)):
             raise ValueError("parent hypotheses must be a list")
 
-        resp = self._rpc.sendAndWait(
+        resp: RPCResponse = self._rpc.sendAndWait(
             command=RPCCommands.addObject,
             args={'object_data': object_data,
                   'parentObjects': parentObjects,
@@ -214,7 +227,8 @@ class ConsoleInterface(object):
 
         return resp
 
-    def addFact(self, fact, creator, require_parentage=True):
+    def _addFact(self, fact: Fact, creator: str,
+                 require_parentage: bool = True) -> RPCResponse:
         """Adds an item to the fact table
 
             Args:
@@ -232,15 +246,16 @@ class ConsoleInterface(object):
             raise ValueError("Fact's parentage must be populated")
 
         fact._creator_ = creator
-        resp = self._rpc.sendAndWait(command=RPCCommands.addFact,
-                                     args={'fact': fact})
+        resp: RPCResponse = self._rpc.sendAndWait(command=RPCCommands.addFact,
+                                                  args={'fact': fact})
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
 
         return resp
 
-    def addHyp(self, hyp, creator, require_parentage=True):
+    def _addHyp(self, hyp: Fact, creator: str,
+                require_parentage: bool = True) -> RPCResponse:
         """Adds an item to the hypothesis table
 
             Args:
@@ -259,8 +274,8 @@ class ConsoleInterface(object):
 
         hyp._creator_ = creator
         hyp._taint()
-        resp = self._rpc.sendAndWait(command=RPCCommands.addHyp,
-                                     args={'hyp': hyp})
+        resp: RPCResponse = self._rpc.sendAndWait(command=RPCCommands.addHyp,
+                                                  args={'hyp': hyp})
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
@@ -286,16 +301,18 @@ class BackStoryConsole(ConsoleInterface):
                          asyncData=kwargs['asyncData'],
                          config=kwargs.get('config'))
 
-        self.__tracker_ = kwargs['tracker']
+        self.__tracker_: Tracker.BackStoryTracker = kwargs['tracker']
 
     @property
-    def memory(self):
+    def memory(self) -> Dict:
         """Property to store backstory-level memory"""
         return self.__tracker_.memory
 
-    def addObject(self, object_data, parentObjects=None,
-                  parentFacts=None, parentHyps=None,
-                  metadata=None, encoding=None):
+    def addObject(self, object_data: bytes,
+                  parentObjects: Optional[List[FileObject]] = None,
+                  parentFacts: Optional[List[Fact]] = None,
+                  parentHyps: Optional[List[Fact]] = None,
+                  metadata: Dict = None, encoding: str = None) -> int:
         """Adds an object to the object list
 
             Args:
@@ -303,13 +320,14 @@ class BackStoryConsole(ConsoleInterface):
 
             Returns: The object id
         """
-        resp = super().addObject(object_data, self.__tracker_.name,
-                                 parentObjects, parentFacts, parentHyps,
-                                 metadata, encoding)
+        resp: RPCResponse = super()._addObject(object_data,
+                                               self.__tracker_.name,
+                                               parentObjects, parentFacts,
+                                               parentHyps, metadata, encoding)
 
         return resp.result.object_id
 
-    def addFact(self, fact):
+    def addFact(self, fact: Fact) -> None:
         """Adds an item to the fact table
 
             Args:
@@ -319,9 +337,9 @@ class BackStoryConsole(ConsoleInterface):
 
             Raises: ValueError if object id and reference are not set
         """
-        super().addFact(fact, self.__tracker_.name, require_parentage=False)
+        super()._addFact(fact, self.__tracker_.name, require_parentage=False)
 
-    def addHyp(self, hyp):
+    def addHyp(self, hyp: Fact) -> None:
         """Adds an item to the hyp table
 
             Args:
@@ -331,7 +349,7 @@ class BackStoryConsole(ConsoleInterface):
 
             Raises: ValueError if object id and reference are not set
         """
-        super().addHyp(hyp, self.__tracker_.name, require_parentage=False)
+        super()._addHyp(hyp, self.__tracker_.name, require_parentage=False)
 
 
 class NPCConsole(ConsoleInterface):
@@ -352,16 +370,18 @@ class NPCConsole(ConsoleInterface):
                          asyncData=kwargs['asyncData'],
                          config=kwargs.get('config'))
 
-        self.__tracker_ = kwargs['tracker']
+        self.__tracker_: Tracker.NPCTracker = kwargs['tracker']
 
     @property
-    def memory(self):
+    def memory(self) -> Dict:
         """Property to store npc-level memory"""
         return self.__tracker_.memory
 
-    def addObject(self, object_data, parentObjects=None,
-                  parentFacts=None, parentHyps=None,
-                  metadata=None, encoding=None):
+    def addObject(self, object_data: bytes,
+                  parentObjects: Optional[List[FileObject]] = None,
+                  parentFacts: Optional[List[Fact]] = None,
+                  parentHyps: Optional[List[Fact]] = None,
+                  metadata: Dict = None, encoding: str = None) -> int:
         """Adds an object to the object list
 
             Args:
@@ -369,13 +389,14 @@ class NPCConsole(ConsoleInterface):
 
             Returns: The object id
         """
-        resp = super().addObject(object_data, self.__tracker_.name,
-                                 parentObjects, parentFacts, parentHyps,
-                                 metadata, encoding)
+        resp: RPCResponse = super()._addObject(object_data,
+                                               self.__tracker_.name,
+                                               parentObjects, parentFacts,
+                                               parentHyps, metadata, encoding)
 
         return resp.result.object_id
 
-    def addFact(self, fact):
+    def addFact(self, fact: Fact) -> None:
         """Adds an item to the fact table
 
             Args:
@@ -385,9 +406,9 @@ class NPCConsole(ConsoleInterface):
 
             Raises: ValueError if object id and reference are not set
         """
-        super().addFact(fact, self.__tracker_.name)
+        super()._addFact(fact, self.__tracker_.name)
 
-    def addHyp(self, hyp):
+    def addHyp(self, hyp: Fact) -> None:
         """Adds an item to the hyp table
 
             Args:
@@ -397,7 +418,7 @@ class NPCConsole(ConsoleInterface):
 
             Raises: ValueError if object id and reference are not set
         """
-        super().addHyp(hyp, self.__tracker_.name)
+        super()._addHyp(hyp, self.__tracker_.name)
 
 
 class PlayerConsole(ConsoleInterface):
@@ -418,82 +439,86 @@ class PlayerConsole(ConsoleInterface):
                          cloneID=kwargs['clone_id'],
                          config=kwargs.get('config', None))
 
-        self.__tainted_ = kwargs['tainted']
-        self.__tracker_ = kwargs['tracker']
+        self.__tainted_: bool = kwargs['tainted']
+        self.__tracker_: Tracker.PlayerTracker = kwargs['tracker']
 
     @property
-    def id(self):
+    def id(self) -> Tuple[str, Optional[str]]:
         return (self._id, self._cloneID)
 
     @property
-    def memory(self):
+    def memory(self) -> Dict:
         return self.__tracker_.memory
 
     @property
-    def __clone_(self):
+    def __clone_(self) -> Optional[str]:
         return self.__tracker_.clones[self._cloneID]
 
     @property
-    def data(self):
+    def data(self) -> bytes:
         return self.__tracker_.cloneMemory[self._cloneID]
 
-    def setWaiting(self):
+    def setWaiting(self) -> None:
         self.__tracker_.clones[self._cloneID] \
             ._state = PlayerState.waiting
 
-    def setRunning(self):
+    def setRunning(self) -> None:
         self.__tracker_.clones[self._cloneID] \
             ._state = PlayerState.running
         self.__tracker_.clones[self._cloneID] \
             ._turnStart = time.time()
 
-    def getClones(self):
+    def getClones(self) -> None:
         pass
 
-    def getObject(self, object_id):
+    def getObject(self, object_id: int):
         """Returns the object with the given id
         """
-        resp = self._rpc.sendAndWait(command=RPCCommands.getObject,
-                                     args={'object_id': object_id})
+        resp: RPCResponse = self._rpc.sendAndWait(
+                                command=RPCCommands.getObject,
+                                args={'object_id': object_id})
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
 
         return resp.result.object
 
-    def getAllObjects(self):
+    def getAllObjects(self) -> List[FileObject]:
         """Returns a list of all objects
         """
-        resp = self._rpc.sendAndWait(command=RPCCommands.getAllObjects)
+        resp: RPCResponse = self._rpc.sendAndWait(
+                                command=RPCCommands.getAllObjects)
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
 
         return resp.result.object_list
 
-    def getFact(self, factID):
+    def getFact(self, factID: int) -> Fact:
         """Returns a specific fact based on id
         """
-        resp = self._rpc.sendAndWait(command=RPCCommands.getFact,
-                                     args={'fact_id': factID})
+        resp: RPCResponse = self._rpc.sendAndWait(
+                                command=RPCCommands.getFact,
+                                args={'fact_id': factID})
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
 
         return resp.result.fact
 
-    def getHyp(self, hypID):
+    def getHyp(self, hypID: int) -> Fact:
         """Returns a specific hypothesis based on id
         """
-        resp = self._rpc.sendAndWait(command=RPCCommands.getHyp,
-                                     args={'hyp_id': hypID})
+        resp: RPCResponse = self._rpc.sendAndWait(
+                                command=RPCCommands.getHyp,
+                                args={'hyp_id': hypID})
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
 
         return resp.result.hyp
 
-    def getAllFacts(self, fact_type):
+    def getAllFacts(self, fact_type: Union[str, List[str]]) -> List[Fact]:
         """Returns a list of all facts of a given type
         """
 
@@ -502,15 +527,16 @@ class PlayerConsole(ConsoleInterface):
 
         fact_type = resolveFacts(*fact_type)
 
-        resp = self._rpc.sendAndWait(command=RPCCommands.getAllFacts,
-                                     args={'fact_type': fact_type})
+        resp: RPCResponse = self._rpc.sendAndWait(
+                                command=RPCCommands.getAllFacts,
+                                args={'fact_type': fact_type})
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
 
         return resp.result.fact_list
 
-    def getAllHyps(self, hyp_type):
+    def getAllHyps(self, hyp_type: Union[str, List[str]]) -> List[Fact]:
         """Returns a list of all hypotheses of a given type
         """
 
@@ -519,15 +545,16 @@ class PlayerConsole(ConsoleInterface):
 
         hyp_type = resolveFacts(*hyp_type)
 
-        resp = self._rpc.sendAndWait(command=RPCCommands.getAllFacts,
-                                     args={'hyp_type': hyp_type})
+        resp: RPCResponse = self._rpc.sendAndWait(
+                                command=RPCCommands.getAllFacts,
+                                args={'hyp_type': hyp_type})
 
         if resp.status == RPCResponseStatus.error:
             raise ConsoleError(resp.reason)
 
         return resp.result.hyp_list
 
-    def _waitOn(self, stream_id):
+    def _waitOn(self, stream_id: int) -> Generator[RPCResponse, None, None]:
         # Set the state to waiting before entering the loop
         # since the generator will block
         self.setWaiting()
@@ -549,7 +576,8 @@ class PlayerConsole(ConsoleInterface):
             # after this function/generator exits
             self.setRunning()
 
-    def waitOnFacts(self, facts, only_latest=False):
+    def waitOnFacts(self, facts: Union[str, List[str]],
+                    only_latest: bool = False) -> Generator[Fact, None, None]:
         """Waits on facts until player breaks out of generator
         """
 
@@ -557,14 +585,16 @@ class PlayerConsole(ConsoleInterface):
             facts = [facts]
         facts = resolveFacts(*facts)
 
-        stream_id = self._rpc.startStream(command=RPCStreamCommands.factStream,
-                                          args={'fact_types': facts,
-                                                'only_latest': only_latest})
+        stream_id: int = self._rpc.startStream(
+                                command=RPCStreamCommands.factStream,
+                                args={'fact_types': facts,
+                                      'only_latest': only_latest})
 
         for msg in self._waitOn(stream_id):
             yield msg.result.fact
 
-    def waitOnHyps(self, hyps, only_latest=False):
+    def waitOnHyps(self, hyps: Union[str, List[str]],
+                   only_latest: bool = False) -> Generator[Fact, None, None]:
         """Waits on hypotheses until player breaks out of generator
         """
 
@@ -572,15 +602,20 @@ class PlayerConsole(ConsoleInterface):
             hyps = [hyps]
         hyps = resolveFacts(*hyps)
 
-        stream_id = self._rpc.startStream(command=RPCStreamCommands.hypStream,
-                                          args={'hyp_types': hyps,
-                                                'only_latest': only_latest})
+        stream_id: int = self._rpc.startStream(
+                                    command=RPCStreamCommands.hypStream,
+                                    args={'hyp_types': hyps,
+                                          'only_latest': only_latest})
 
         for msg in self._waitOn(stream_id):
             yield msg.result.hyp
 
-    def waitOnChildFacts(self, object_id=None, fact_id=None, hyp_id=None,
-                         facts=None, only_latest=False):
+    def waitOnChildFacts(self, object_id: Optional[int] = None,
+                         fact_id: Optional[int] = None,
+                         hyp_id: Optional[int] = None,
+                         facts: Union[str, List[str]] = None,
+                         only_latest: bool = False
+                         ) -> Generator[Fact, None, None]:
         """Wait on facts of given types, for a given object id, fact id, or
             hyp id until player breaks out of generator
         """
@@ -600,7 +635,7 @@ class PlayerConsole(ConsoleInterface):
             facts = [facts]
         facts = resolveFacts(*facts)
 
-        stream_id = self._rpc.startStream(
+        stream_id: int = self._rpc.startStream(
             command=RPCStreamCommands.childFactStream,
             args={'object_id': object_id,
                   'fact_id': fact_id,
@@ -611,8 +646,12 @@ class PlayerConsole(ConsoleInterface):
         for msg in self._waitOn(stream_id):
             yield msg.result.fact
 
-    def waitOnChildHyps(self, object_id=None, fact_id=None, hyp_id=None,
-                        types=None, only_latest=False):
+    def waitOnChildHyps(self, object_id: Optional[int] = None,
+                        fact_id: Optional[int] = None,
+                        hyp_id: Optional[int] = None,
+                        types: Union[str, List[str]] = None,
+                        only_latest: bool = False
+                        ) -> Generator[Fact, None, None]:
         """Wait on hypotheses of given types, for a given object id, fact id,
             or hypothesis id until player breaks out of generator
         """
@@ -632,7 +671,7 @@ class PlayerConsole(ConsoleInterface):
             types = [types]
         types = resolveFacts(*types)
 
-        stream_id = self._rpc.startStream(
+        stream_id: int = self._rpc.startStream(
             command=RPCStreamCommands.childHypStream,
             args={'object_id': object_id,
                   'fact_id': fact_id,
@@ -643,8 +682,11 @@ class PlayerConsole(ConsoleInterface):
         for msg in self._waitOn(stream_id):
             yield msg.result.hyp
 
-    def waitOnChildObjects(self, object_id=None, fact_id=None,
-                           hyp_id=None, only_latest=False):
+    def waitOnChildObjects(self, object_id: Optional[int] = None,
+                           fact_id: Optional[int] = None,
+                           hyp_id: Optional[int] = None,
+                           only_latest: bool = False
+                           ) -> Generator[FileObject, None, None]:
         """Wait on child objects of a given object id, fact id, or
             hypothesis id
         """
@@ -657,7 +699,7 @@ class PlayerConsole(ConsoleInterface):
             raise ValueError(("Only one of object id, fact_id, or hyp_id "
                               "maybe used"))
 
-        stream_id = self._rpc.startStream(
+        stream_id: int = self._rpc.startStream(
             command=RPCStreamCommands.childObjectStream,
             args={'object_id': object_id,
                   'fact_id': fact_id,
@@ -667,7 +709,9 @@ class PlayerConsole(ConsoleInterface):
         for msg in self._waitOn(stream_id):
             yield msg.result.object
 
-    def waitTillFact(self, fact_type, last_fact=None, timeout=0):
+    def waitTillFact(self, fact_type: Union[str, List[str]],
+                     last_fact: Optional[int] = None,
+                     timeout: int = 0) -> None:
         """Waits up till timeout (default forever) for fact to show up
 
             If timeout is reached will return None
@@ -677,12 +721,15 @@ class PlayerConsole(ConsoleInterface):
 
         fact_type = resolveFacts(*fact_type)
 
-        msg_id = self._rpc.sendMessage(command=RPCCommands.waitTillFact,
-                                       args={'fact_type': fact_type,
-                                             'last_fact': last_fact})
+        msg_id: int = self._rpc.sendMessage(command=RPCCommands.waitTillFact,
+                                            args={'fact_type': fact_type,
+                                                  'last_fact': last_fact})
 
         self.setWaiting()
-        resp = self._rpc.waitForResponse(msg_id, timeout)
+        try:
+            resp = self._rpc.waitForResponse(msg_id, timeout)
+        except RPCTimeoutError:
+            pass
         self.setRunning()
 
         if resp is not None:
@@ -691,21 +738,23 @@ class PlayerConsole(ConsoleInterface):
             raise WaitTimeoutError()
             # TODO FIXME XXX
 
-    def addObject(self, object_data, parentObjects=None,
-                  parentFacts=None, parentHyps=None,
-                  metadata=None, encoding=None):
+    def addObject(self, object_data: bytes,
+                  parentObjects: Optional[List[FileObject]] = None,
+                  parentFacts: Optional[List[Fact]] = None,
+                  parentHyps: Optional[List[Fact]] = None,
+                  metadata: Dict = None, encoding: str = None) -> int:
         """Adds an object to the object list
 
             returns the object id
         """
-        resp = super().addObject(object_data,
-                                 self.__tracker_.name,
-                                 parentObjects, parentFacts, parentHyps,
-                                 metadata, encoding)
+        resp = super()._addObject(object_data,
+                                  self.__tracker_.name,
+                                  parentObjects, parentFacts, parentHyps,
+                                  metadata, encoding)
 
         return resp.result.object_id
 
-    def addFact(self, fact, yesreally=False):
+    def addFact(self, fact: Fact, yesreally: bool = False) -> None:
         """Adds an item to the fact table
 
             Args:
@@ -720,9 +769,9 @@ class PlayerConsole(ConsoleInterface):
             raise ValueError(("Adding a fact based on a hypothesis, requires "
                               "the 'yesreally' argument to be set to True"))
 
-        super().addFact(fact, self.__tracker_.name)
+        super()._addFact(fact, self.__tracker_.name)
 
-    def addHyp(self, hyp):
+    def addHyp(self, hyp: Fact) -> None:
         """Adds an item to the hyp table
 
             Args:
@@ -734,4 +783,4 @@ class PlayerConsole(ConsoleInterface):
         """
 
         hyp._taint()
-        super().addHyp(hyp, self.__tracker_.name)
+        super()._addHyp(hyp, self.__tracker_.name)
