@@ -3,59 +3,65 @@ import queue
 import threading
 from argparse import Namespace
 from enum import Enum
+from typing import (Callable, Optional, Dict, Union, Set,
+                    List, Tuple, Type, Generator)
 
-from d20.Manual.Logger import logging
-from d20.Manual.Exceptions import StreamTimeoutError
+from d20.Manual.Logger import logging, Logger
+from d20.Manual.Exceptions import StreamTimeoutError, RPCTimeoutError
 
 
-LOGGER = logging.getLogger(__name__)
+LOGGER: Logger = logging.getLogger(__name__)
 
 
 class RPCResponseStatus(Enum):
-    ok = 1
-    error = 2
+    ok: int = 1
+    error: int = 2
 
 
 class RPCCommands(Enum):
-    print = 0
-    addObject = 1
-    addFact = 2
-    getObject = 3
-    getAllObjects = 4
-    getFact = 5
-    getAllFacts = 6
-    startStream = 7
-    stopStream = 8
-    waitTillFact = 9
-    addHyp = 10
-    getHyp = 11
-    getAllHyps = 12
-    promote = 13
-    noop = 14
+    print: int = 0
+    addObject: int = 1
+    addFact: int = 2
+    getObject: int = 3
+    getAllObjects: int = 4
+    getFact: int = 5
+    getAllFacts: int = 6
+    startStream: int = 7
+    stopStream: int = 8
+    waitTillFact: int = 9
+    addHyp: int = 10
+    getHyp: int = 11
+    getAllHyps: int = 12
+    promote: int = 13
+    noop: int = 14
 
 
 class RPCStreamCommands(Enum):
-    factStream = 1
-    childFactStream = 2
-    childObjectStream = 3
-    hypStream = 4
-    childHypStream = 5
+    factStream: int = 1
+    childFactStream: int = 2
+    childObjectStream: int = 3
+    hypStream: int = 4
+    childHypStream: int = 5
 
 
 class EntityType(Enum):
-    npc = 0
-    player = 1
-    backstory = 2
+    npc: int = 0
+    player: int = 1
+    backstory: int = 2
 
 
 class Entity:
-    def __init__(self, entity_type, rpc_id, id, clone_id=None):
-        self.entity_type = entity_type
-        self.rpcClient = rpc_id
-        self.id = id
-        self.clone = clone_id
+    def __init__(self, entity_type: EntityType, rpc_id: int, id: int,
+                 clone_id: Optional[int] = None) -> None:
+        self.entity_type: EntityType = entity_type
+        self.rpcClient: int = rpc_id
+        self.id: int = id
+        self.clone: Optional[int] = clone_id
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, Entity):
+            return False
 
-    def __eq__(self, other):
         if self.entity_type != other.entity_type:
             return False
 
@@ -67,13 +73,13 @@ class Entity:
 
         return True
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         return not self.__eq__(other)
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.isNPC:
             return "npc-%d" % (self.id)
-        elif self.isPlayer:
+        elif self.isPlayer and self.clone is not None:
             return "player-%d-clone-%d" % (self.id, self.clone)
         elif self.isBackStory:
             return "backstory-%d" % (self.id)
@@ -81,31 +87,32 @@ class Entity:
             return "unknown-%d" % (self.id)
 
     @property
-    def isPlayer(self):
+    def isPlayer(self) -> bool:
         if self.entity_type == EntityType.player:
             return True
         return False
 
     @property
-    def isNPC(self):
+    def isNPC(self) -> bool:
         if self.entity_type == EntityType.npc:
             return True
         return False
 
     @property
-    def isBackStory(self):
+    def isBackStory(self) -> bool:
         if self.entity_type == EntityType.backstory:
             return True
         return False
 
 
 class RPCStream:
-    def __init__(self, command, args=None):
+    def __init__(self, command: RPCStreamCommands,
+                 args: Optional[Union[Dict, Namespace]] = None) -> None:
         if not isinstance(command, RPCStreamCommands):
             raise TypeError("Expected an RPCStreamCommands value for command")
 
-        self.command = command
-        self.args = None
+        self.command: RPCStreamCommands = command
+        self.args: Optional[Union[Dict, Namespace]] = None
         if args is not None:
             if isinstance(args, dict):
                 self.args = Namespace(**args)
@@ -116,18 +123,19 @@ class RPCStream:
 
 
 class RPCRequest:
-    __msg_counter_ = 0
+    __msg_counter_: int = 0
 
-    def __init__(self, entity, command, args=None):
+    def __init__(self, entity: Entity, command: RPCCommands,
+                 args: Optional[Union[Dict, Namespace]] = None) -> None:
         if not isinstance(entity, Entity):
             raise TypeError("Expected an instance of 'Entity' for entity")
-        self.entity = entity
+        self.entity: Entity = entity
 
         if not isinstance(command, RPCCommands):
             raise TypeError("Expected an RPCCommands value for command")
-        self.command = command
+        self.command: RPCCommands = command
 
-        self.args = None
+        self.args: Optional[Union[Dict, Namespace]] = None
         if args is not None:
             if isinstance(args, dict):
                 self.args = Namespace(**args)
@@ -136,41 +144,44 @@ class RPCRequest:
             else:
                 raise TypeError("Expected a dict or Namespace type for args")
 
-        self.id = RPCRequest.generateMsgId()
+        self.id: int = RPCRequest.generateMsgId()
 
     @staticmethod
-    def generateMsgId():
+    def generateMsgId() -> int:
         RPCRequest.__msg_counter_ += 1
         return RPCRequest.__msg_counter_
 
 
 class RPCStartStreamRequest(RPCRequest):
-    def __init__(self, entity, command, args=None):
+    def __init__(self, entity: Entity, command: RPCStreamCommands,
+                 args: Optional[Union[Dict, Namespace]] = None) -> None:
         super().__init__(entity, RPCCommands.startStream)
         self.stream = RPCStream(command, args=args)
 
 
 class RPCStopStreamRequest(RPCRequest):
-    def __init__(self, entity, stream_id):
+    def __init__(self, entity: Entity, stream_id: int) -> None:
         super().__init__(entity, RPCCommands.stopStream,
                          args={'stream_id': stream_id})
 
 
 class RPCResponse:
-    def __init__(self, id, status, result=None, reason=None):
-        self.id = id
+    def __init__(self, id: int, status: RPCResponseStatus,
+                 result: Optional[Union[Namespace, Dict]] = None,
+                 reason: Optional[str] = None) -> None:
+        self.id: int = id
 
         if not isinstance(status, RPCResponseStatus):
             raise TypeError("status must be a RPCResponseStatus type")
 
-        self.status = status
+        self.status: RPCResponseStatus = status
         if status == RPCResponseStatus.error:
             if reason is None:
                 raise ValueError(("Reason must be populated when status is "
                                   "set to 'error'"))
             self.reason = reason
 
-        self.result = None
+        self.result: Namespace
         if result is not None:
             if isinstance(result, dict):
                 self.result = Namespace(**result)
@@ -181,34 +192,35 @@ class RPCResponse:
 
 
 class RPCClient:
-    client_id = 0
+    client_id: int = 0
 
-    def __init__(self, server_queue, entity_type, entity_id, clone_id=None):
-        self.server_queue = server_queue
-        self.client_queue = queue.Queue()
-        self.id = RPCClient.generateClientId()
-        self.entity = Entity(entity_type, self.id, entity_id, clone_id)
-        self.msglist = dict()
-        self.streams = dict()
-        self.timeouts = set()
-        self.ignores = set()
-        self._stop = False
-        self._respThread = threading.Thread(
+    def __init__(self, server_queue: queue.Queue, entity_type: EntityType,
+                 entity_id: int, clone_id: Optional[int] = None) -> None:
+        self.server_queue: queue.Queue = server_queue
+        self.client_queue: queue.Queue = queue.Queue()
+        self.id: int = RPCClient.generateClientId()
+        self.entity: Entity = Entity(entity_type, self.id, entity_id, clone_id)
+        self.msglist: Dict[int, RPCResponse] = dict()
+        self.streams: Dict[int, queue.Queue] = dict()
+        self.timeouts: Set = set()
+        self.ignores: Set = set()
+        self._stop: bool = False
+        self._respThread: threading.Thread = threading.Thread(
             target=self.respThread,
             name='rpcClient.%d' % self.id)
         self._respThread.daemon = True
         self._respThread.start()
 
-    def stop(self):
+    def stop(self) -> None:
         self._stop = True
 
-    def join(self):
+    def join(self) -> None:
         self._respThread.join()
 
-    def respThread(self):
+    def respThread(self) -> None:
         while not self._stop:
             try:
-                resp = self.client_queue.get_nowait()
+                resp: RPCResponse = self.client_queue.get_nowait()
             except queue.Empty:
                 time.sleep(.01)
                 continue
@@ -221,7 +233,7 @@ class RPCClient:
                 LOGGER.error("Received malformed response")
                 continue
 
-            msg_id = resp.id
+            msg_id: int = resp.id
 
             # If this request timed out, ignore it
             if msg_id in self.timeouts:
@@ -238,10 +250,11 @@ class RPCClient:
             else:
                 self.msglist[msg_id] = resp
 
-    def startStream(self, command, args=None):
-        request = RPCStartStreamRequest(self.entity,
-                                        command,
-                                        args=args)
+    def startStream(self, command: RPCStreamCommands,
+                    args: Optional[Dict] = None) -> int:
+        request: RPCStartStreamRequest = RPCStartStreamRequest(self.entity,
+                                                               command,
+                                                               args=args)
 
         self.streams[request.id] = queue.Queue()
 
@@ -250,12 +263,12 @@ class RPCClient:
         self.server_queue.put(request)
         return request.id
 
-    def stopStream(self, stream_id):
+    def stopStream(self, stream_id: int) -> None:
         if stream_id not in self.streams.keys():
             raise RuntimeError("Attempt to stop untracked stream")
 
-        request = RPCStopStreamRequest(self.entity,
-                                       stream_id)
+        request: RPCStopStreamRequest = RPCStopStreamRequest(self.entity,
+                                                             stream_id)
         # Remove stream information
         del self.streams[stream_id]
 
@@ -263,13 +276,15 @@ class RPCClient:
         self.ignores.add(request.id)
         self.server_queue.put(request)
 
-    def getStream(self, stream_id, timeout=None):
+    def getStream(self, stream_id: int,
+                  timeout: Optional[int] = None
+                  ) -> Generator[RPCResponse, None, None]:
         if stream_id not in self.streams.keys():
             raise RuntimeError("Attempt to get untracked stream")
 
         while 1:
             try:
-                msg = self.streams[stream_id].get(timeout=timeout)
+                msg: RPCResponse = self.streams[stream_id].get(timeout=timeout)
                 yield msg
             except queue.Empty:
                 raise StreamTimeoutError()
@@ -278,25 +293,29 @@ class RPCClient:
                                   "for stream data"))
                 raise StopIteration()
 
-    def sendMessage(self, command, args, ignore=False):
-        request = RPCRequest(entity=self.entity, command=command, args=args)
+    def sendMessage(self, command: RPCCommands,
+                    args: Optional[Union[Dict, Namespace]],
+                    ignore: bool = False) -> int:
+        request: RPCRequest = RPCRequest(entity=self.entity, command=command,
+                                         args=args)
 
         if ignore:
             self.ignores.add(request.id)
         self.server_queue.put(request)
         return request.id
 
-    def waitForResponse(self, msg_id, timeout=0):
+    def waitForResponse(self, msg_id: int,
+                        timeout: int = 0) -> RPCResponse:
         start = time.time()
         while 1:
             if timeout > 0 and time.time() - start > timeout:
                 self.timeouts.add(msg_id)
-                return None
+                raise RPCTimeoutError()
             if msg_id not in self.msglist:
                 time.sleep(.01)
                 continue
 
-            resp = self.msglist[msg_id]
+            resp: RPCResponse = self.msglist[msg_id]
             del self.msglist[msg_id]
 
             if (not isinstance(resp, RPCResponse) or
@@ -305,38 +324,41 @@ class RPCClient:
 
             return resp
 
-    def sendAndIgnore(self, command, args=None):
-        msg_id = self.sendMessage(command, args, ignore=True)
+    def sendAndIgnore(self, command: RPCCommands,
+                      args: Optional[Union[Dict, Namespace]] = None) -> int:
+        msg_id: int = self.sendMessage(command, args, ignore=True)
         return msg_id
 
-    def sendAndWait(self, command, args=None, timeout=0):
-        msg_id = self.sendMessage(command, args)
+    def sendAndWait(self, command: RPCCommands,
+                    args: Optional[Union[Dict, Namespace]] = None,
+                    timeout: int = 0) -> RPCResponse:
+        msg_id: int = self.sendMessage(command, args)
         return self.waitForResponse(msg_id, timeout)
 
-    def handleResponse(self, response):
+    def handleResponse(self, response: RPCResponse) -> None:
         self.client_queue.put(response)
 
     @staticmethod
-    def generateClientId():
+    def generateClientId() -> int:
         RPCClient.client_id += 1
         return RPCClient.client_id
 
 
 class RPCServer:
     def __init__(self):
-        self.server_queue = queue.Queue()
-        self.clients = dict()
-        self.streams = dict()
-        self._stop = False
-        self.handlers = dict()
-        self.startStreamHandlers = dict()
-        self.stopStreamHandlers = dict()
-        self.idleFn = None
-        self.rpc_thread = threading.Thread(
+        self.server_queue: queue.Queue = queue.Queue()
+        self.clients: Dict[int, RPCClient] = dict()
+        self.streams: Dict = dict()
+        self._stop: bool = False
+        self.handlers: Dict[RPCCommands, Callable] = dict()
+        self.startStreamHandlers: Dict[RPCStreamCommands, Callable] = dict()
+        self.stopStreamHandlers: Dict[RPCStreamCommands, Callable] = dict()
+        self.idleFn: Callable = None
+        self.rpc_thread: threading.Thread = threading.Thread(
             target=self.runGame,
             name='rpcServer')
 
-    def registerHandler(self, command, fn):
+    def registerHandler(self, command: RPCCommands, fn: Callable) -> None:
         if not callable(fn):
             raise TypeError("Provided function doesn't look callable")
         if not isinstance(command, RPCCommands):
@@ -349,11 +371,13 @@ class RPCServer:
         LOGGER.debug("Registering Handler for %s procedure" % command)
         self.handlers[command] = fn
 
-    def registerHandlers(self, handlers):
+    def registerHandlers(self,
+                         handlers: List[Tuple[RPCCommands, Callable]]) -> None:
         for (command, fn) in handlers:
             self.registerHandler(command, fn)
 
-    def registerStartStreamHandler(self, command, fn):
+    def registerStartStreamHandler(self, command: RPCStreamCommands,
+                                   fn: Callable) -> None:
         if not callable(fn):
             raise TypeError("Provided function doesn't look callable")
         if not isinstance(command, RPCStreamCommands):
@@ -362,7 +386,8 @@ class RPCServer:
                      % command)
         self.startStreamHandlers[command] = fn
 
-    def registerStopStreamHandler(self, command, fn):
+    def registerStopStreamHandler(self, command: RPCStreamCommands,
+                                  fn: Callable) -> None:
         if not callable(fn):
             raise TypeError("Provided function doesn't look callable")
         if not isinstance(command, RPCStreamCommands):
@@ -371,34 +396,38 @@ class RPCServer:
                      % command)
         self.stopStreamHandlers[command] = fn
 
-    def registerStreamHandler(self, command, start_fn, stop_fn):
+    def registerStreamHandler(self, command: RPCStreamCommands,
+                              start_fn: Callable, stop_fn: Callable) -> None:
         self.registerStartStreamHandler(command, start_fn)
         self.registerStopStreamHandler(command, stop_fn)
 
-    def registerStreamHandlers(self, handlers):
+    def registerStreamHandlers(self,
+                               handlers: List[Tuple[RPCStreamCommands,
+                                                    Callable,
+                                                    Callable]]) -> None:
         for (command, start_fn, stop_fn) in handlers:
             self.registerStreamHandler(command, start_fn, stop_fn)
 
-    def registerIdleFunction(self, fn):
+    def registerIdleFunction(self, fn: Callable) -> None:
         if not callable(fn):
             raise TypeError("Provided function doesn't look callable")
         LOGGER.debug("Registering Idle Function")
         self.idleFn = fn
 
-    def start(self):
+    def start(self) -> None:
         self.rpc_thread.start()
 
-    def isAlive(self):
+    def isAlive(self) -> bool:
         return self.rpc_thread.isAlive()
 
-    def join(self):
+    def join(self) -> None:
         return self.rpc_thread.join()
 
-    def runGame(self):
-        start = time.time()
+    def runGame(self) -> None:
+        start: float = time.time()
         while not self._stop:
             try:
-                msg = self.server_queue.get_nowait()
+                msg: Type[RPCRequest] = self.server_queue.get_nowait()
                 start = time.time()
                 self._idleCount = 0
             except queue.Empty:
@@ -428,11 +457,13 @@ class RPCServer:
                              % (msg.command, msg.id, tstring, msg.entity.id))
 
             try:
-                command = msg.command
+                command: RPCCommands = msg.command
 
-                if command == RPCCommands.startStream:
+                if isinstance(msg, RPCStartStreamRequest) and \
+                        command == RPCCommands.startStream:
                     self.startStream(msg)
-                elif command == RPCCommands.stopStream:
+                elif isinstance(msg, RPCStopStreamRequest) and \
+                        command == RPCCommands.stopStream:
                     self.stopStream(msg)
                 else:
                     if command not in self.handlers:
@@ -443,55 +474,62 @@ class RPCServer:
             except Exception:
                 LOGGER.exception("Issue running command")
 
-    def stop(self):
+    def stop(self) -> None:
         self._stop = True
 
-    def startStream(self, msg):
-        command = msg.stream.command
+    def startStream(self, msg: RPCStartStreamRequest) -> None:
+        command: RPCStreamCommands = msg.stream.command
         if command not in self.startStreamHandlers:
             raise RuntimeError("No start handler defined for stream function")
 
         self.startStreamHandlers[command](msg)
         self.streams[msg.id] = msg
 
-    def stopStream(self, msg):
-        stream_id = msg.args.stream_id
-        if stream_id not in self.streams:
-            pass  # TODO FIXME return error or something
+    def stopStream(self, msg: RPCStopStreamRequest) -> None:
+        if isinstance(msg.args, Namespace):
+            stream_id = msg.args.stream_id
+            if stream_id not in self.streams:
+                pass  # TODO FIXME return error or something
 
-        request = self.streams[stream_id]
-        command = request.stream.command
+            request = self.streams[stream_id]
+            command = request.stream.command
 
-        if command not in self.stopStreamHandlers:
-            raise RuntimeError("No stop handler defined for stream function")
+            if command not in self.stopStreamHandlers:
+                raise RuntimeError("No stop handler defined for stream \
+                                    function")
 
-        self.stopStreamHandlers[command](msg)
+            self.stopStreamHandlers[command](msg)
 
-        del self.streams[request.id]
-        self.sendResponse(msg, RPCResponseStatus.ok)
+            del self.streams[request.id]
+            self.sendResponse(msg, RPCResponseStatus.ok)
 
-    def createClient(self, entity_type, entity_id, clone_id=None):
+    def createClient(self, entity_type: EntityType, entity_id: int,
+                     clone_id: Optional[int] = None) -> RPCClient:
         LOGGER.debug("Generating RPC Client")
-        client = RPCClient(self.server_queue, entity_type, entity_id, clone_id)
+        client: RPCClient = RPCClient(self.server_queue, entity_type,
+                                      entity_id, clone_id)
         self.clients[client.id] = client
         return client
 
-    def destroyClient(self, client_id):
+    def destroyClient(self, client_id: int) -> None:
         LOGGER.debug("Destroying RPC client %d" % (client_id))
         client = self.clients[client_id]
         client.stop()
         client.join()
         del self.clients[client_id]
 
-    def sendResponse(self, msg, status, result=None, reason=None):
+    def sendResponse(self, msg: RPCRequest, status: RPCResponseStatus,
+                     result: Optional[Union[Namespace, Dict]] = None,
+                     reason: Optional[str] = None) -> None:
         if not isinstance(msg, RPCRequest):
             raise TypeError("Expected an RPCRequest")
-        response = RPCResponse(msg.id, status, result, reason)
+        response: RPCResponse = RPCResponse(msg.id, status, result, reason)
         client_id = msg.entity.rpcClient
         self.clients[client_id].handleResponse(response)
 
-    def sendErrorResponse(self, msg, reason):
+    def sendErrorResponse(self, msg: RPCRequest, reason: str) -> None:
         self.sendResponse(msg, RPCResponseStatus.error, reason=reason)
 
-    def sendOKResponse(self, msg, result=None):
+    def sendOKResponse(self, msg: RPCRequest, result:
+                       Optional[Union[Namespace, Dict]] = None) -> None:
         self.sendResponse(msg, RPCResponseStatus.ok, result=result)
